%* lsaf_getprocessflows                                                                            *;
%*                                                                                                 *;
%* Gets the metadata for all of the process flows defined at, and optionally the children of, the  *;
%* specified context. The metadata is stored in a SAS data set.                                    *;
%*                                                                                                 *;
%* macrotype LSAF                                                                                  *;
%* since 1.5                                                                                       *;
%* exposure external                                                                               *;
%*                                                                                                 *;
%* History                                                                                         *;
%* 2016-08-17  original creation.                                                                  *;
%* 2019-02-12  update to 2.1                                                                       *;
%* 2019-11-01  update to handle a blank SAS output file.                                           *;
%* 2021-01-22  add processFlowId column to sas_dsname.                                             *;
%* 2021-08-12  Remove header documentation                                                         *;
%* 2021-09-20  Remove parameter defaults (DE16775)                                                 *;
%* 2021-10-13  US11739                                                                             *;
%*                                                                                                 *;

%macro lsaf_getprocessflows(
  lsaf_path=,
  sas_dsname=work.lsafGetProcessFlows,
  lsaf_includeChildren=
  ) /des='Get a list of process flows in a SAS Life Science Analytics Framework context.';
  
  %if "&sas_dsname" EQ "" %then %let sas_dsname=work.lsafGetProcessFlows;

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %local _lsafRowCount_;

  %let _lsafRowCount_=0;

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data &sas_dsname (label="Created by &SYSMACRONAME"
                    keep=contextPath name description createdBy created dateCreated lastModifiedBy lastModified dateLastModified
                         owner status processDefinitionId processDefinitionVersion processFlowId);

    attrib returnCode                length=8
           message                   length=$200
           logMsgPrefix              length=$70
           logMsgPrimary             length=$70
           logMsg                    length=$300
           rowCount                  length=8

           contextPath               length=$2048  label="Context Path"
           name                      length=$256   label="Process Flow Name"
           description               length=$1024  label="Process Flow Description"
           createdBy                 length=$64    label="Userid that created the process flow"
           created                   length=$64    label="Process Flow Creation Date"
           dateCreated                             label="Date the Process Flow was created (SAS DateTime Format)"
           lastModifiedBy            length=$64    label="Userid that last modified the process flow"
           lastModified              length=$64    label="Last Modification Date"
           dateLastModified                        label="Date the process flow was last modified (SAS DateTime Format)"
           owner                     length=$64    label="Userid of process flow owner"
           status                    length=$64    label="Status of the Process Flow"
           processDefinitionId       length=$256   label="Name of the Process Flow Definition used to create the Process Flow"
           processDefinitionVersion  length=$32    label="Version Number of the Process Flow Definition used to create the Process Flow"
           processFlowId             length=$32    label="Process Flow ID"

           createdTempSAS            length=$64    label="TEMPORARY - WILL BE DROPPED"
           lastModifiedTempSAS       length=$64    label="TEMPORARY - WILL BE DROPPED"
    ;

    format dateCreated
           dateLastModified datetime.
    ;

    rowCount=0;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/workflow/processflow/ProcessFlowService") ;
    declare javaobj results("com/sas/lsaf/macro/workflow/processflow/SasProcessFlowsResult");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("getProcessFlows", "%bquote(&lsaf_path)", "%bquote(&lsaf_includeChildren)", results) ;

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot retrieve process flows for &lsaf_path" ;
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" Process flows were returned for &lsaf_path" ;
      results.callIntMethod("getSize", rowCount);
      do row=0 to rowCount - 1;
        results.callStringMethod("getContextPath", row, contextPath);
        results.callStringMethod("getName", row, name);
        results.callStringMethod("getDescription", row, description);
        results.callStringMethod("getCreated", row, created);
        results.callStringMethod("getCreatedSasFormat", row, createdTempSas);
        results.callStringMethod("getCreatedBy", row, createdBy);
        results.callStringMethod("getLastModified", row, lastModified);
        results.callStringMethod("getLastModifiedSasFormat", row, lastModifiedTempSas);
        results.callStringMethod("getLastModifiedBy", row, lastModifiedBy);
        results.callStringMethod("getStatus", row, status);
        results.callStringMethod("getOwner", row, owner);
        results.callStringMethod("getProcessDefinitionId", row, processDefinitionId);
        results.callStringMethod("getProcessDefinitionVersion", row, processDefinitionVersion);
        results.callStringMethod("getId", row, processFlowId);
        dateCreated=input(createdTempSAS, datetime.);
        dateLastModified=input(lastModifiedTempSas, datetime.);
        output;
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put "NOTE: ***************************************************************************************";
    put "NOTE: The &SYSMACRONAME. macro will be deprecated in a future release of the SAS Life        ";
    put "NOTE: Science Analytics Framework and replaced with the lsaf_getprocessflowsbyquery macro.   ";
    put "NOTE: ***************************************************************************************";
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    if (rowCount=0) 
      then put logMsgPrefix " No process flows were found.";
      else put logMsgPrefix " " rowCount " records written to data set &sas_dsname";
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafRowCount_", rowCount);

  run;

  %***********************************;
  %* Sort by name                    *;
  %***********************************;
  %if (&_lsafRowCount_ > 0) %then
  %do;
    %let notesOption=%sysfunc(getoption(notes));
    options nonotes;
    proc sort data=&sas_dsname sortseq=linguistic(strength=primary);
      by name;
    run;
    options &notesOption;
  %end;

%mend lsaf_getprocessflows;
